package model;

import java.text.Collator;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

/**
 * Třída pro jeden dokument, větu nebo hledaný řetězec
 *
 * @author Matěj Fries
 */
public class Dokument
{

    /**
     * List slov jak byl dokument zadán
     */
    private List<String> puvodniZneni;
    /**
     * Název dokumentu
     */
    private String nazev;
    /**
     * List slov v dokumentu
     */
    private List<String> slova = new ArrayList<>();
    /**
     * List počtů výskytů slov
     */
    private List<Integer> pocet = new ArrayList<>();

    /**
     * Konstruktor dokumentu, vytvoří list slov a list počtů jejich výskytů
     *
     * @param seznam list slov v dokumentu
     * @param nazev název dokumentu
     */
    public Dokument(List<String> seznam, String nazev)
    {
        puvodniZneni = new ArrayList<>();
        this.nazev = nazev;
        String slovo;
        String pom;
        boolean pridano = false;

        for (int i = 0; i < seznam.size(); i++)
        {
            boolean povolen = true;

            slovo = seznam.get(i);
            slovo = slovo.replaceAll("\\s+", "");

            if (!slovo.isEmpty())
            {
                if (slovo.length() == 1)
                {
                    if (!Character.isDigit(slovo.charAt(0)) && !Character.isLetter(slovo.charAt(0)))
                    {
                        povolen = false;
                    }
                }
            } else
            {
                povolen = false;
            }

            if (povolen)
            {
                puvodniZneni.add(slovo);
                if (!slova.isEmpty())
                {
                    for (int j = 0; j < slova.size(); j++)
                    {
                        pom = slova.get(j);
                        if (slovo.equals(pom))
                        {
                            pocet.set(j, pocet.get(j) + 1);
                            pridano = true;
                            break;
                        }
                    }
                    if (pridano)
                    {
                        pridano = false;
                    } else
                    {
                        slova.add(slovo);
                        pocet.add(1);
                    }
                } else
                {
                    slova.add(slovo);
                    pocet.add(1);
                }
            }
        }
    }

    /**
     * Vrací list počtů výskytů slov v dokumentu
     *
     * @return list počtů výskytů slov
     */
    public List<Integer> getPocet()
    {
        return pocet;
    }

    /**
     * Vrací list slov, jak byl dokument zadán
     *
     * @return list slov dokumentu
     */
    public List<String> getPuvodniZneni()
    {
        return puvodniZneni;
    }

    /**
     * Vrací list počtů výskytů v dokumentu ve formátu double
     *
     * @return list počtů výskytu slov
     */
    public List<Double> getPocetDouble()
    {
        List<Double> pocetDouble = new ArrayList<>();
        for (int i = 0; i < pocet.size(); i++)
        {
            pocetDouble.add((double) pocet.get(i));
        }

        return pocetDouble;
    }

    /**
     * Vrací vektor s počty výskytů slov v dokumentu
     *
     * @return vektor s počty výskytů slov
     */
    public Vektor getPocetVektor()
    {
        Vektor v = new Vektor(this.getPocetDouble());
        return v;
    }

    /**
     * Vrací list se slovy vyskytujícími se v dokumentu
     *
     * @return list se slovy
     */
    public List<String> getSlova()
    {
        return slova;
    }

    /**
     * Seřadí slova v dokumentu a jejich počty podle počtů jejich výskytů
     */
    public void seradPodlePoctu()
    {
        if (pocet != null && pocet.size() > 1)
        {
            serad(0, pocet.size() - 1, 1);
        }
    }

    /**
     * Seřadí slova v dokumentu a jejich počty podle abecedního pořadí slov
     */
    public void seradPodleAbecedy()
    {
        if (slova != null && slova.size() > 1)
        {
            serad(0, pocet.size() - 1, 2);
        }
    }

    /**
     * Privární metoda pro seřazení slov a počtů jejich výskytů
     *
     * @param l levý index
     * @param p pravý index
     * @param podle podle čeho se seznam řadí (1 - podle počtů, 2 - podle
     * abecedy)
     */
    private void serad(int l, int p, int podle)
    {
        Collator por = Collator.getInstance(new Locale("cs", "CZ"));
        por.setStrength(Collator.PRIMARY);

        if (l < p)
        {
            int b = l;
            for (int i = l + 1; i < p; i++)
            {
                switch (podle)
                {
                    case 1:
                        if (Double.compare(pocet.get(i), pocet.get(l)) > 0)
                        {
                            prohod(i, ++b);
                        }
                        break;
                    case 2:
                        if (por.compare(slova.get(i), slova.get(l)) < 0)
                        {
                            prohod(i, ++b);
                        }
                        break;
                    default:
                        break;
                }

            }
            prohod(l, b);
            serad(l, b, podle);
            serad(b + 1, p, podle);
        }
    }

    /**
     * Privátn metoda pro prohození dvou hodnot (prohodí list se slovy i s počty
     * výskytů)
     *
     * @param x první hodnota
     * @param y druhá hodnota
     */
    private void prohod(int x, int y)
    {
        int pomP = pocet.get(x);
        String pomS = slova.get(x);

        pocet.set(x, pocet.get(y));
        slova.set(x, slova.get(y));

        pocet.set(y, pomP);
        slova.set(y, pomS);
    }

    /**
     * Vrací název dokumentu
     *
     * @return název dokumentu
     */
    public String getNazev()
    {
        return nazev;
    }

    /**
     * Natavuje název dokumentu
     *
     * @param nazev nový název dokumentu
     */
    public void setNazev(String nazev)
    {
        this.nazev = nazev;
    }
}
